#################################################################################
## R function to calculate overlap of two raster files with the same extent
##
## WARNING: this function is a dependency of "nicheModelingPresent.ProjectionPast"
##           it uses objects specified or generated by such function
##			 objects required: paths to save PDF files, lat/long extent, speciesName, gps.
##			
##
## Notes: To calculate overlap of two raster files with different extents see: 
## 			http://stackoverflow.com/questions/17930336/calculating-area-of-overlap-between-2-raster-layers-with-different-origins-and-e
## 			review http://gis.stackexchange.com/questions/140504/extracting-intersection-areas-in-r
##
## R dependencies: raster, maps.
##
## Author: Ariadna Morales [ariadna.biologia@gmail.com]
##
## 			Before using this version look at https://github.com/ariadnamorales
## 			there might be an updated version of this function.
##
## Last saved version: May 26th, 2017
##
#################################################################################

##------> Arguments
# d: raster object   -> current climatic conditions
# b: raster object   -> lgm (climatic conditions in the past)
# threshold_probSDM: value betweeen 0 and 1000 representing the probability of ocurrence of SDM 
# MIN_lon=low_lon     -----> value estimated during calculations using function nicheModelingPresent.ProjectionPast 
# MAX_lon=high_lon   -----> value estimated during calculations using function nicheModelingPresent.ProjectionPast
# MIN_lat=low_lat 	  -----> value estimated during calculations using function nicheModelingPresent.ProjectionPast
# MAX_lat=high_lat	  -----> value estimated during calculations using function nicheModelingPresent.ProjectionPast
# gpsPoints=gps       -----> dataFrame with gps points

##------------------------> start function
calculateAreaOverlap<-function(speciesName, d, b, threshold_probSDM, MIN_lon, MAX_lon, MIN_lat, MAX_lat, gpsPoints){

	## Load libraries
	print("Loading required libraries")
	library(raster)
	library(maps)

	print(paste("--------Starting calculations with threshold ",threshold_probSDM,"--------",sep=""))
		## replace values of 'd' raster matrix to 0 and 1 based on 'threshold_probSDM'
		# current
		print("Replacing values of 'd' based on 'threshold_probSDM'")
		d@data@values[is.na(d@data@values)] <- NA
		## considerably faster than loops
		d_values<-d@data@values
		d_values<-lapply(d_values, function(x){replace(x, x<threshold_probSDM, NA)})
		d_values<-lapply(d_values, function(x){replace(x, x>=threshold_probSDM, 1)})
		d_values<-unlist(d_values)
		d@data@values<-d_values
			
		## replace values of 'b' raster matrix to 0 and 1 based on 'threshold_probSDM'
		# LGM
		print("Replacing values of 'b' based on 'threshold_probSDM'")
		b@data@values[is.na(b@data@values)] <- NA
		## considerably faster than loops
		b_values<-b@data@values
		b_values<-lapply(b_values, function(x){replace(x, x<threshold_probSDM, NA)})
		b_values<-lapply(b_values, function(x){replace(x, x>=threshold_probSDM, 1)})
		b_values<-unlist(b_values)
		b@data@values<-b_values
		
		
		## intersect layers
		y <- intersect(b, d)
		print("Intersection done")

		## define colors for maps 
		lm.palette_d <- colorRampPalette(c(rgb(49,130,189,100,maxColorValue=255)), space = "rgb")			#----->blue
		lm.palette_b <- colorRampPalette(c(rgb(227,74,51,100,maxColorValue=255)), space = "rgb")				#----->red
		lm.palette_y <- colorRampPalette(c(rgb(129,15,124,100,maxColorValue=255)), space = "rgb")			## ----> shared 

		## Save map projection of area of overlapping
		pdf(paste(getwd(),"/",gsub("_", ".", speciesName),"/maps/",gsub("_", ".", speciesName),"_mapOverlap_C-LGM_t",threshold_probSDM,"_.pdf",sep=""))
			image(b, col = lm.palette_b(1), xlab="longitude", ylab="latitude", main=bquote(italic(.(speciesName))), xlim = c(MIN_lon, MAX_lon), ylim = c(MIN_lat, MAX_lat))
			image(d, col = lm.palette_d(1), add = TRUE)
			image(y, col = lm.palette_y(1), legend=FALSE, add = TRUE)
			points(gpsPoints$Longitude, gpsPoints$Latitude, col="blue", pch=20, cex=1)
			map(xlim = c(MIN_lon, MAX_lon), ylim = c(MIN_lat, MAX_lat), add=TRUE)
			mtext(paste("overlap Present-LGM thr=",threshold_probSDM/10,"%",sep=""), side=3)
		dev.off()
		print("Map of projection saved")

		## calculate shared area
		print("Calculating areas of overlapping")
		area_P<-sum(values(d), na.rm=TRUE)		# Area in the present
		area_LGM<-sum(values(b), na.rm=TRUE)	# Area in the LGM
		area_P_LGM<-sum(values(y), na.rm=TRUE)  # overlapped area

			## add threshold value to area variables
			## rename variables
			assign(paste('area_',paste("P_t",threshold_probSDM,sep=""),sep=""),area_P)
			assign(paste('area_',paste("LGM_t",threshold_probSDM,sep=""),sep=""),area_LGM)
			assign(paste('areaShared_',paste("P.LGM_t",threshold_probSDM,sep=""),sep=""),area_P_LGM)

			## merge variables in a Table
			areasTable<-as.data.frame(cbind(get(paste("area_P_t",threshold_probSDM,sep="")), get(paste("area_LGM_t",threshold_probSDM,sep="")), 	get(paste("areaShared_P.LGM_t",threshold_probSDM,sep=""))))

			## add headers to table
			colNames_areaTable<-c(paste("area_P_t",threshold_probSDM,sep=""), paste("area_LGM_t",threshold_probSDM,sep=""), paste("areaShared_P.LGM_t",threshold_probSDM,sep=""))
			colnames(areasTable)[1]<-colNames_areaTable[1]
			colnames(areasTable)[2]<-colNames_areaTable[2]
			colnames(areasTable)[3]<-colNames_areaTable[3]

		# Save table of areas
		write.table(areasTable, file = paste(getwd(),"/",gsub("_", ".", speciesName),"/area_t",threshold_probSDM,"_",gsub("_", ".", speciesName),".txt",sep=""), col.names=TRUE, row.names=FALSE, quote=FALSE, sep = "\t")
		print("Tables saved")
		print(paste("--------END of calculations with threshold ",threshold_probSDM,"--------",sep=""))

}
###------------------------> END function